package org.expeditee.items;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Point;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.SwingUtilities;

import org.expeditee.gui.Browser;
import org.expeditee.gui.DisplayIO;
import org.expeditee.gui.Frame;
import org.expeditee.gui.FrameGraphics;

/**
 * The bridge between swing space and expeditee space
 * 
 * @author Brook
 * 
 */
public abstract class InteractiveWidget {

	protected JComponent _component;

	/** A widget is comprised of dots and lines that basically form a rectangle */
	private WidgetCorner _d1, _d2, _d3, _d4;

	private WidgetEdge _l1, _l2, _l3, _l4;

	/* GUIDE:
	 * 			 l1
	 * 		d1-------d2
	 * 		|         |
	 *   l4 |    X    | 12
	 * 		|         |
	 * 		d4-------d3
     *           13
	 */
	private List<Item> _items; // used for quickly returning item list

	// Widget size restrictions
	private int _minWidth = 50;

	private int _minHeight = 50;

	private int _maxWidth = 300;

	private int _maxHeight = 300;

	// The Expeditee item that is used for saving widget state in expiditee
	// world
	private Text _source;

	// A flag for signifying whether the swing components are ready to paint.
	// If the swing components has not been layed out, if they are painted they
	// will
	// not draw in the correct positions.
	private boolean _isReadyToPaint = false;

	/**
	 * Creates a InteractiveWidget from a text item.
	 * 
	 * @param source
	 *            Must not be null, first line of text used - which the format
	 *            must be as follows: "@iw <<widget_class_name>> [<<width>>] [<<height>>] [: [<<arg1>>] [<<arg2>>]
	 *            [...]]".
	 * 
	 * e.g: "@iw org.expeditee.items.SampleWidget1 100 20 : 2" creates a
	 * SampleWidget1 with width = 100 and height = 20 with 1 argument = "2"
	 * 
	 * @return An InteractiveWidget instance. Never null.
	 * 
	 * @throws NullPointerException
	 *             if source is null
	 * 
	 * @throws IllegalArgumentException
	 *             if source's text is in the incorrect format or if source's
	 *             parent is null
	 * 
	 * @throws InteractiveWidgetNotAvailableException
	 *             If the given widget class name in the source text doesn't
	 *             exist or not an InteractiveWidget or the widget does not
	 *             supply a valid constructor for creation.
	 * 
	 * Note: could throw other exceptions depending on the type of widget being
	 * instantainiated. source must have a parent
	 * 
	 * class names beginning with $, the $ will be replaced with
	 * "org.expeditee.items."
	 */
	public static InteractiveWidget CreateWidget(Text source)
			throws InteractiveWidgetNotAvailableException {

		if (source == null)
			throw new NullPointerException("source");
		if (source.getParent() == null)
			throw new IllegalArgumentException(
					"source's parent is null, InteractiveWidget's must be created from Text items with non-null parents");

		String TAG = ItemUtils.GetTag(ItemUtils.TAG_IWIDGET);

		String text = source.getTextNoList();
		if (text == null)
			throw new IllegalArgumentException("source does not have any text");

		int index = text.indexOf(':'); // used for signifying start of
										// arguments
		if (index == 0)
			throw new IllegalArgumentException("Source text must begin with \""
					+ TAG + "\"");

		String[] tokens = (index == -1) ? text.split("\\s+") : text.substring(
				0, index).split(" ");

		// Check starts with the widget tag
		if (tokens.length < 1 || !tokens[0].equals(TAG))
			throw new IllegalArgumentException("Source text must begin with \""
					+ TAG + "\"");

		int width = -1, height = -1;

		if (tokens.length < 2)
			throw new IllegalArgumentException(
					"Missing widget class name in source text");

		try {

			if (tokens.length >= 3) { // parse optional width
				width = Integer.parseInt(tokens[2]);
				width = (width <= 0) ? width = -1 : width;
			}

			if (tokens.length >= 4) { // parse optional height
				height = Integer.parseInt(tokens[3]);
				height = (height <= 0) ? height = -1 : height;
			}

		} catch (NumberFormatException nfe) {
			throw new IllegalArgumentException(
					"Bad width or height given in source text", nfe);
		}

		if (tokens.length > 4)
			throw new IllegalArgumentException(
					"to many arguments given before \":\" in source text");

		String classname = tokens[1];
		if (classname.charAt(0) == '$')
			classname = "org.expeditee.items." + classname.substring(1);
		// Attempt to locate the class using reflection
		Class<?> iwclass = findIWidgetClass(classname);

		if (iwclass == null) // ensure it exists
			throw new InteractiveWidgetNotAvailableException(classname
					+ " does not exit or is not an InteractiveWidget");

		// Extract out the parameters - if any
		String[] args = null;
		if (index > 0) { // index of the first ":"
			args = (text.length() == (index + 1)) ? null : parseArgs(text
					.substring(index + 1));
		}

		InteractiveWidget inst = null;
		try {
			// Instantainiate the widget - passing the params
			Class parameterTypes[] = new Class[] { Text.class, String[].class };
			Constructor ct = iwclass.getConstructor(parameterTypes);

			Object arglist[] = new Object[] { source, args };

			inst = (InteractiveWidget) ct.newInstance(arglist);
		} catch (Exception e) {
			throw new InteractiveWidgetNotAvailableException(
					"Failed to create instance via reflection: " + e.toString(),
					e);
		}

		// Use default dimensions if not provided (or provided as negitive
		// values)
		if (width <= 0)
			width = inst.getWidth();
		if (height <= 0)
			height = inst.getHeight();

		inst.setSize(width, height);

		return inst;
	}

	/**
	 * Locates the class from the classname of an InteractiveWidget class
	 * 
	 * @param classname
	 *            The name of the class to search
	 * @return Null if doesn't exist or not an InteractiveWidget
	 */
	private static Class findIWidgetClass(String classname) {
		try {
			Class c = Class.forName(classname); // attempt to find the class

			// If one is found, ensure that it is a descendant of an
			// InteractiveWidget
			for (Class superclass = c.getSuperclass(); superclass != null
					&& superclass != Item.class; superclass = superclass
					.getSuperclass()) {
				if (superclass == InteractiveWidget.class)
					return c;
			}

		} catch (ClassNotFoundException e) {
		}

		// Doesn't exist or not an InteractiveWidget
		return null;
	}

	/**
	 * Using Microsofts commandline convention: Args seperated with white
	 * spaces. Options with white spaces enclosed with quotes. Args with quotes
	 * must be double quoted args1 args2=sfasas args3="option with spaces"
	 * arg""4""
	 * 
	 * @param args
	 *            Null and empty excepted
	 * @return An array of args. null if none provided
	 */
	static String[] parseArgs(String args) {

		if (args == null)
			return null;

		args = args.trim();
		if (args.length() == 0)
			return null;

		List<String> vargs = new LinkedList<String>();
		StringBuilder sb = new StringBuilder();
		boolean quoteOn = false;
		for (int i = 0; i < args.length(); i++) {

			char c = args.charAt(i);
			if (c == ' ' && !quoteOn) {

				// Extract arg
				vargs.add(sb.toString());
				sb = new StringBuilder();

				// Consume white spaces
				while (args.charAt(++i) == ' ' && i < args.length()) {
				}
				i--;

			} else if (c == '\"') {
				// If double qouted
				if (args.length() >= (i + 2) && args.charAt(i + 1) == '\"') {

					sb.append(c); // add escaped
					i++;

				} else {
					quoteOn = !quoteOn;
				}

			} else {
				sb.append(c);
			}
		}

		if (sb.length() > 0)
			vargs.add(sb.toString());

		if (vargs.size() == 0)
			return null;
		else
			return vargs.toArray(new String[vargs.size()]);
	}

	/**
	 * Reverse of parseArgs
	 * 
	 * @return Null if args is null or empty / all whitespace
	 */
	static String formatArgs(String[] args) {
		if (args == null)
			return null;

		StringBuilder sb = new StringBuilder();

		for (String s : args) {
			if (s == null)
				continue;

			// Escape quotes
			StringBuilder formatted = new StringBuilder(s.replaceAll("\"",
					"\"\""));

			// Encapsulate spaces
			int index = formatted.indexOf(" ");
			if (index >= 0) {
				formatted.insert(index, "\"");
				formatted.append('\"');
			}

			if (sb.length() > 0)
				sb.append(' ');
			sb.append(formatted);
		}

		return sb.length() > 0 ? sb.toString() : null;
	}

	/**
	 * Arguments represent state. They are used for saving, loading, creating
	 * and cloning Special formatting is done for you.
	 * 
	 * @param isCopy
	 *            If true, then the requested arguments returned should
	 *            represent a copy of this instance Otherwise the arguments
	 *            returned should represent this instance. This gives the widget
	 *            the chance to allocate new/duplicate resources if they need
	 *            to.
	 * 
	 * @return Can be null for no params.
	 */
	protected abstract String[] getArgs(boolean isCopy);

	/**
	 * Constructor
	 * 
	 * @param source
	 *            Must not be null. Neither must it's parent
	 * @param component
	 *            Must not be null
	 */
	protected InteractiveWidget(Text source, JComponent component,
			int minWidth, int maxWidth, int minHeight, int maxHeight) {

		if (component == null)
			throw new NullPointerException("component");
		if (source == null)
			throw new NullPointerException("source");
		if (source.getParent() == null)
			throw new IllegalArgumentException(
					"source's parent is null, InteractiveWidget's must be created from Text items with non-null parents");

		_component = component;
		_source = source;

		_minWidth = minWidth;

		if (maxWidth < _minWidth && maxWidth >= 0)
			throw new IllegalArgumentException(
					"maxWidth smaller than the min Width");
		_maxWidth = maxWidth;

		_minHeight = minHeight;
		if (maxHeight < _minHeight && maxHeight >= 0)
			throw new IllegalArgumentException(
					"maxHeight smaller than the min Height");
		_maxHeight = maxHeight;

		int x = source.getX();
		int y = source.getY();
		int width = (_minWidth < 0) ? 10 : _minWidth;
		int height = (_minHeight < 0) ? 10 : _minHeight;

		Frame idAllocator = _source.getParent();

		// create WidgetCorners
		_d1 = new WidgetCorner(x, y, idAllocator.getNextItemID(), this);
		_d2 = new WidgetCorner(x + width, y, idAllocator.getNextItemID(), this);
		_d3 = new WidgetCorner(x + width, y + height, idAllocator
				.getNextItemID(), this);
		_d4 = new WidgetCorner(x, y + height, idAllocator.getNextItemID(), this);

		// create WidgetEdges
		_l1 = new WidgetEdge(_d1, _d2, idAllocator.getNextItemID());
		_l2 = new WidgetEdge(_d2, _d3, idAllocator.getNextItemID());
		_l3 = new WidgetEdge(_d3, _d4, idAllocator.getNextItemID());
		_l4 = new WidgetEdge(_d4, _d1, idAllocator.getNextItemID());

		_items = new ArrayList<Item>(8); // Note: order important
		_items.add(_d1);
		_items.add(_d2);
		_items.add(_d3);
		_items.add(_d4);
		_items.add(_l1);
		_items.add(_l2);
		_items.add(_l3);
		_items.add(_l4);
	}

	/**
	 * @return A copy of this widget
	 */
	public final InteractiveWidget copy()
			throws InteractiveWidgetNotAvailableException {

		Text t = _source.copy();
		String clonedAnnotation = getAnnotationString(true);
		_source.setText(clonedAnnotation);
		return InteractiveWidget.CreateWidget(t);
	}

	/**
	 * @return The top-most class name including the package.
	 */
	protected abstract String getTopMostClassName();

	/**
	 * Note updates the source text with current state info
	 * 
	 * @return The Text item that this widget was created from.
	 */
	public Text getSource() {

		// Build the annotation string such that it represents this widgets
		// current state
		String newAnnotation = getAnnotationString(false);

		// Set the new text
		_source.setText(newAnnotation);

		return _source;
	}

	/**
	 * @param copiedVersion
	 *            True to return a copied representation of this instance. False
	 *            to return this instances current state version.
	 * 
	 * @return The expeditee anotation string.
	 */
	private String getAnnotationString(boolean copiedVersion) {

		// Create tag and append classname
		StringBuilder sb = new StringBuilder(ItemUtils
				.GetTag(ItemUtils.TAG_IWIDGET));
		sb.append(' ');
		sb.append(getTopMostClassName());

		// Append size information if needed (not an attibute of text items)
		if (!isFixedSize()) {
			sb.append(' ');
			sb.append(getWidth());
			sb.append(' ');
			sb.append(getHeight());
		}

		// Append arguments if any
		String stateArgs = InteractiveWidget.formatArgs(getArgs(copiedVersion));
		if (stateArgs != null) {
			sb.append(':');
			sb.append(stateArgs);
		}

		return sb.toString();
	}

	/**
	 * Clamped to current min/max width/height.
	 * 
	 * @param width
	 * @param height
	 */
	public void setSize(int width, int height) {

		// Clamp
		if (width < _minWidth && _minWidth >= 0)
			width = _minWidth;
		else if (width > _maxWidth && _maxWidth >= 0)
			width = _maxWidth;

		if (height < _minHeight && _minHeight >= 0)
			height = _minHeight;
		else if (height > _maxHeight && _maxHeight >= 0)
			height = _maxHeight;

		boolean vfloating[] = new boolean[] { _d1.isFloating(),
				_d2.isFloating(), _d3.isFloating(), _d4.isFloating() };

		_d1.setFloating(true);
		_d2.setFloating(true);
		_d3.setFloating(true);
		_d4.setFloating(true);

		int x = _d1.getX() + width;
		int y = _d1.getY() + height;
		_d2.setX(x);
		_d3.setX(x);
		_d3.setY(y);
		_d4.setY(y);

		_d1.setFloating(vfloating[0]);
		_d2.setFloating(vfloating[1]);
		_d3.setFloating(vfloating[2]);
		_d4.setFloating(vfloating[3]);

	}

	public void setPosition(int x, int y) {

		boolean vfloating[] = new boolean[] { _d1.isFloating(),
				_d2.isFloating(), _d3.isFloating(), _d4.isFloating() };

		int width = getWidth();
		int height = getHeight();

		_d1.setFloating(true);
		_d2.setFloating(true);
		_d3.setFloating(true);
		_d4.setFloating(true);

		_d1.setPosition(x, y);
		_d2.setPosition(x + width, y);
		_d3.setPosition(x + width, y + height);
		_d4.setPosition(x, y + height);

		_d1.setFloating(vfloating[0]);
		_d2.setFloating(vfloating[1]);
		_d3.setFloating(vfloating[2]);
		_d4.setFloating(vfloating[3]);

	}

	private boolean _settingPositionFlag = false; // used for recursion

	/**
	 * to be called from corners only
	 * 
	 * @param src
	 * @param x
	 * @param y
	 * @return True if need to call super.setPosition
	 */
	boolean setPositions(WidgetCorner src, int x, int y) {

		if (_settingPositionFlag)
			return false;
		_settingPositionFlag = true;

		// Check to see if the widget is fully being picked up
		boolean isAllPickedUp = (_d1.isFloating() && _d2.isFloating()
				&& _d3.isFloating() && _d4.isFloating());

		int newX = x;

		// X Positions
		if (src == _d1 || src == _d4) {

			if (src == _d1 && isAllPickedUp)
				_d1.setX(newX);
			else if (src == _d4 && isAllPickedUp)
				_d4.setX(newX);
			else {
				// Check min X constraint
				if (_minWidth >= 0) {
					if ((_d2.getX() - x) < _minWidth) {
						newX = _d2.getX() - _minWidth;
					}
				}
				// Check max X constraint
				if (_maxWidth >= 0) {
					if ((_d2.getX() - x) > _maxWidth) {
						newX = _d2.getX() - _maxWidth;
					}
				}

				if (!(src == _d4 && _d1.isFloating() && _d4.isFloating()))
					_d1.setX(newX);
				if (!(src == _d1 && _d4.isFloating() && _d1.isFloating()))
					_d4.setX(newX);
			}

		} else if (src == _d2 || src == _d3) {

			if (src == _d2 && isAllPickedUp)
				_d2.setX(newX);
			else if (src == _d3 && isAllPickedUp)
				_d3.setX(newX);
			else {
				// Check min X constraint
				if (_minWidth >= 0) {
					if ((x - _d1.getX()) < _minWidth) {
						newX = _d1.getX() + _minWidth;
					}
				}
				// Check max X constraint
				if (_maxWidth >= 0) {
					if ((x - _d1.getX()) > _maxWidth) {
						newX = _d1.getX() + _maxWidth;
					}
				}

				if (!(src == _d3 && _d2.isFloating() && _d3.isFloating()))
					_d2.setX(newX);
				if (!(src == _d2 && _d3.isFloating() && _d2.isFloating()))
					_d3.setX(newX);
			}

		}

		int newY = y;

		// Y Positions
		if (src == _d1 || src == _d2) {

			if (src == _d1 && isAllPickedUp)
				_d1.setY(newY);
			else if (src == _d2 && isAllPickedUp)
				_d2.setY(newY);
			else {
				// Check min Y constraint
				if (_minHeight >= 0) {
					if ((_d4.getY() - y) < _minHeight) {
						newY = _d4.getY() - _minHeight;
					}
				}
				// Check max Y constraint
				if (_maxHeight >= 0) {
					if ((_d4.getY() - y) > _maxHeight) {
						newY = _d4.getY() - _maxHeight;
					}
				}

				if (!(src == _d2 && _d1.isFloating() && _d2.isFloating()))
					_d1.setY(newY);
				if (!(src == _d1 && _d2.isFloating() && _d1.isFloating()))
					_d2.setY(newY);

			}

		} else if (src == _d3 || src == _d4) {

			if (src == _d3 && isAllPickedUp)
				_d3.setY(newY);
			else if (src == _d4 && isAllPickedUp)
				_d4.setY(newY);
			else {
				// Check min Y constraint
				if (_minHeight >= 0) {
					if ((y - _d1.getY()) < _minHeight) {
						newY = _d1.getY() + _minHeight;
					}
				}
				// Check max Y constraint
				if (_maxHeight >= 0) {
					if ((y - _d1.getY()) > _maxHeight) {
						newY = _d1.getY() + _maxHeight;
					}
				}

				if (!(src == _d4 && _d3.isFloating() && _d4.isFloating()))
					_d3.setY(newY);
				if (!(src == _d3 && _d4.isFloating() && _d3.isFloating()))
					_d4.setY(newY);
			}
		}

		// Update source text position so position is remembered from loading
		int newTextX = getX();
		int newTextY = getY();
		if (_source.getX() != newTextX || _source.getY() != newTextY)
			_source.setPosition(newTextX, newTextY);

		_settingPositionFlag = false;
		return true;
	}

	public int getX() {
		return Math.min(_d1.getX(), _d2.getX());
	}

	public int getY() {
		return Math.min(_d1.getY(), _d4.getY());
	}

	public int getWidth() {
		return Math.abs(_d2.getX() - _d1.getX());
	}

	public int getHeight() {
		return Math.abs(_d4.getY() - _d1.getY());
	}

	/**
	 * The order of the items in the list is as specified: _d1 _d2 _d3 _d4 _l1
	 * _l2 _l3 _l4
	 * 
	 * @return All of the Expeditee items that form the bounderies of this
	 *         widget in an unmodifiable list
	 */
	public List<Item> getItems() {
		return Collections.unmodifiableList(_items);
	}

	public JComponent getComponant() {
		return _component;
	}

	public final void onParentFameHidden(WidgetCorner notifier) {
		onRemovedFromFrame(notifier);
	}

	public final void onParentFameShown(WidgetCorner notifier,
			boolean isOverlayed, int overlayLevel) {
		addJComponantToFrame(notifier, isOverlayed, overlayLevel, true);
	}

	public void onRemovedFromFrame(WidgetCorner notifier) {
		if (notifier == _d4) {
			if (_component.getParent() != null) {
				_component.getParent().remove(_component);
			}
		}
	}

	public void onAddedToFrame(WidgetCorner notifier, boolean isOverlayed,
			int overlayLevel) {
		addJComponantToFrame(notifier, isOverlayed, overlayLevel, false);
	}

	private void addJComponantToFrame(WidgetCorner notifier,
			boolean isOverlayed, int overlayLevel, boolean isShown) {
		if (notifier == _d4) {
			if (isOverlayed && overlayLevel == Item.PERMISSION_NONE)
				return; // non-active overlays

			if (_component.getParent() == null) {
				if (Browser._theBrowser != null) {
					// Due to precaching - before adding physical swing
					// componant must check
					// to see that this widget belongs to a frame that is
					// considered current.
					// If the widget is shown however this does not apply -
					// since it has been explicitly
					// made clear the the widget is shown.
					if (isShown
							|| _d1.getParent() == DisplayIO.getCurrentFrame()) {
						Browser._theBrowser.getContentPane().add(_component);
						layout(_component);
					}

				} else { // if widgets exist on startup frame this will occur

					synchronized (_widgetsToAddLater) {
						_widgetsToAddLater.add(new WidgetInfo(notifier,
								isOverlayed, overlayLevel, this));
					}
					SwingUtilities.invokeLater(new AddToFrameLater());
				}
			}
		}
	}

	/**
	 * Used for passing info to the swing thread
	 * 
	 * @author Brook Novak
	 */
	class WidgetInfo {

		WidgetInfo(WidgetCorner notifier, boolean isOverlayed,
				int overlayLevel, InteractiveWidget widget) {
			_widget = widget;
			_isOverlayed = isOverlayed;
			_overlayLevel = overlayLevel;
			_notifier = notifier;
		}

		InteractiveWidget _widget;

		boolean _isOverlayed;

		int _overlayLevel;

		WidgetCorner _notifier;
	}

	/**
	 * Must be able to add widgets on first loaded frame: these are loaded
	 * before the browser singleton is made available.
	 */
	private static List<WidgetInfo> _widgetsToAddLater = new LinkedList<WidgetInfo>();

	/**
	 * Ensures widgets are added correctly to first loaded frame
	 */
	class AddToFrameLater implements Runnable {
		public void run() {
			if (!_widgetsToAddLater.isEmpty()) {
				List<WidgetInfo> tmp = null;
				synchronized (_widgetsToAddLater) {
					tmp = new LinkedList<WidgetInfo>(_widgetsToAddLater);
				}
				_widgetsToAddLater.clear();
				for (WidgetInfo iwi : tmp) {
					iwi._widget.onAddedToFrame(iwi._notifier, iwi._isOverlayed,
							iwi._overlayLevel);
				}
			}
		}
	}

	/**
	 * Due to absolute positioning...
	 * 
	 * @param parent
	 */
	private void layout(Component parent) {

		parent.doLayout();

		if (parent instanceof Container) {
			for (Component c : ((Container) parent).getComponents()) {

				if (c instanceof Container)
					layout(c);
				else
					c.doLayout();
			}
		}

		_isReadyToPaint = true;
	}

	public void onBoundsChanged() {

		_component.setBounds(getX(), getY(), getWidth(), getHeight());

	}

	/**
	 * Paints the widget excluding the boundries. That is, the Swing graphics
	 * 
	 * @param g
	 */
	public void paint(Graphics g) {

		if (!_isReadyToPaint) {
			layout(_component);
		}

		Point loc = _component.getLocation();
		g.translate(loc.x, loc.y - 1);
		_component.paint(g);
		g.translate(-loc.x, -(loc.y - 1));

	}

	/**
	 * Called from the widgets corners: Whenever the corners are repainted and
	 * the widget is floating (e.g. currently picked up / rubberbanding) then a
	 * shaded area is drawn instead of the actual widget so the manipulation of
	 * the widget is as smooth as possible. The shaded area is only draw once so
	 * it selects one of the 4 possible notifiers to be the actual trigger to
	 * draw the shaded area
	 * 
	 * @param g
	 * @param notifier
	 */
	void paint(Graphics g, WidgetCorner notifier) {
		if (_component.getParent() == null) {
			// Note that frames with @f may want to paint the widgets so do not
			// paint over the widget interface in these cases: must only
			// paint if an object is floating
			if (notifier == _d4
					&& (_d1.isFloating() || _d2.isFloating()
							|| _d3.isFloating() || _d4.isFloating())) {
				g.setColor(Color.DARK_GRAY);
				g.fillRect(getX(), getY(), getWidth(), getHeight());
			}
		}
	}

	/**
	 * @return True if this widget cannot be resized in either directions
	 */
	public boolean isFixedSize() {
		return this._minHeight == this._maxHeight
				&& this._minWidth == this._maxWidth && this._minHeight >= 0
				&& this._minWidth >= 0;
	}

	private static boolean _isExepiteeHighlightingEnabled = true;

	/**
	 * Enables/Disables highlighting of all widgets - if the enable value has
	 * changed the current frame will repaint.
	 * 
	 * The purpose of this is so that widgets can have an extra context: no
	 * expeditee highlighting if focus is on a swing componet.
	 * 
	 * @param enable
	 *            Set to true to enable painting, false to disable painting.
	 */
	public static void enableExepiteeHighlighting(boolean enable) {
		if (_isExepiteeHighlightingEnabled != enable) {
			_isExepiteeHighlightingEnabled = enable;
			FrameGraphics.Repaint();
		}
	}

	public static boolean isExepiteeHighlightingEnabled() {
		return _isExepiteeHighlightingEnabled;
	}

}
